/*
 *  Part of ase2rv source code,
 *  Copyright (C) 2000 Alexander Kroeller (alilein@gmx.de)
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
#ifndef __ASE_MESH_H
#define __ASE_MESH_H

#include <rvtypes.h>
#include <mesh.h>
#include "aseparser.h"
#include "asevlist.h"
#include "asefacevlist.h"


void dump_sg_tables(void);


enum WorkMode
{
  None, Morph, Ali
};

struct ASEFace
{
  int vert_idx_3ds[3];
  rvulong sm_bits;
  int mat_id;
};

struct ASEFaceNormalSection
{
  int face_id;
  int vert_id[3];
  Vector vert_normal[3];

  ASEFaceNormalSection *next;
};

class ASEMesh
{
public:
  ASEMesh();
  ~ASEMesh();

  void parse(ASEParser &p);

  void checkSanity(void);

  Mesh *createMesh(int master_mat_id, WorkMode wmode);

private:
  void deleteVectorList(ASEVectorList &);
  void setNumFaces3ds(int);
  void readFaceList(ASEParser &);
  void readFaceVertices(ASEParser &, ASEFace *);
  void readNormalList(ASEParser &);
  void createPoly(int, Mesh *, int &);

  void createPolyGeom(int, Mesh *, int &, int master_mat, WorkMode wmode);
  void applyPolyVColors(Mesh *);
  void applyPolyVTextures(Mesh *);
  void applyFaceNormal(Mesh *, ASEFaceNormalSection *);

  int num_faces_3ds;

  ASEFace **faces_3ds;
  ASEFaceNormalSection *first_normal, *last_normal;

  ASEVectorList *verts_3ds;
  ASEVectorList *tvert_3ds;
  ASEVectorList *cvert_3ds;

  ASEFaceVList *tface_3ds;
  ASEFaceVList *cface_3ds;
};


#endif
